=== modified file 'lib/canonical/launchpad/browser/librarian.py'
--- lib/canonical/launchpad/browser/librarian.py	2011-04-27 04:16:20 +0000
+++ lib/canonical/launchpad/browser/librarian.py	2011-06-29 11:21:47 +0000
@@ -14,6 +14,7 @@
     ]
 
 from lazr.delegates import delegates
+from lazr.restful.interfaces import IWebBrowserOriginatingRequest
 from zope.publisher.interfaces import NotFound
 from zope.security.interfaces import Unauthorized
 
@@ -29,7 +30,7 @@
 from canonical.launchpad.webapp.url import urlappend
 from canonical.lazr.utils import get_current_browser_request
 from canonical.librarian.client import url_path_quote
-from lazr.restful.interfaces import IWebBrowserOriginatingRequest
+from lp.app.errors import GoneError
 
 
 class LibraryFileAliasView(LaunchpadView):
@@ -45,13 +46,9 @@
         # Refuse to serve restricted files. We're not sure that no
         # restricted files are being leaked in the traversal hierarchy.
         assert not self.context.restricted
-        # Perhaps we should give a 404 at this point rather than asserting?
-        # If someone has a page open with an attachment link, then someone
-        # else deletes the attachment, this is a normal situation, not an
-        # error. -- RBC 20100726.
-        assert not self.context.deleted, (
-            "LibraryFileAliasView can not operate on deleted librarian files,"
-            " since their URL is undefined.")
+        # If the LFA is deleted, throw a 410.
+        if self.context.deleted:
+            raise GoneError("File deleted.")
         # Redirect based on the scheme of the request, as set by
         # Apache in the 'X-SCHEME' environment variable, which is
         # mapped to 'HTTP_X_SCHEME.  Note that only some requests

=== added file 'lib/canonical/launchpad/browser/tests/test_libraryfilealiasview.py'
--- lib/canonical/launchpad/browser/tests/test_libraryfilealiasview.py	1970-01-01 00:00:00 +0000
+++ lib/canonical/launchpad/browser/tests/test_libraryfilealiasview.py	2011-06-29 11:28:52 +0000
@@ -0,0 +1,30 @@
+# Copyright 2011 Canonical Ltd.  This software is licensed under the
+# GNU Affero General Public License version 3 (see the file LICENSE).
+
+"""Test LibraryFileAliasView."""
+
+__metaclass__ = type
+
+from zope.component import getMultiAdapter
+from zope.security.proxy import removeSecurityProxy
+
+from canonical.launchpad.webapp.servers import LaunchpadTestRequest
+from canonical.testing.layers import LaunchpadFunctionalLayer
+from lp.app.errors import GoneError
+from lp.testing import TestCaseWithFactory
+
+
+class TestLibraryFileAliasView(TestCaseWithFactory):
+    layer = LaunchpadFunctionalLayer
+
+    def test_deleted_lfa(self):
+        # When we initialise a LibraryFileAliasView against a deleted LFA,
+        # we throw a 410 Gone error.
+        lfa = self.factory.makeLibraryFileAlias()
+        removeSecurityProxy(lfa).content = None
+        self.assertTrue(lfa.deleted)
+        request = LaunchpadTestRequest(
+            environ={'REQUEST_METHOD': 'GET', 'HTTP_X_SCHEME' : 'http' })
+        view = getMultiAdapter((lfa, request), name='+index')
+        self.assertRaisesWithContent(
+            GoneError, "'File deleted.'", view.initialize)

